==> scrapy-redis-expiredupefilter-0.2.0.tar.gz: scrapy-redis-expiredupefilter-0.2.0/src/scrapy_redis_expiredupefilter/dupefilter.py <==
"""

"""

import logging
import time

from scrapy.dupefilters import BaseDupeFilter
from scrapy.utils.request import request_fingerprint

from . import defaults
from .connection import get_redis_from_settings

logger = logging.getLogger(__name__)


# TODO: Rename class to RedisDupeFilter.
class RFPDupeFilter(BaseDupeFilter):
    """Redis-based request duplicates filter.

    This class can also be used with default Scrapy's scheduler.

    """

    logger = logger

    def __init__(self, server, key, expire_time, debug=False):
        """Initialize the duplicates filter.

        Parameters
        ----------
        server : redis.StrictRedis
            The redis server instance.
        key : str
            Redis key Where to store fingerprints.
        debug : bool, optional
            Whether to log filtered requests.

        """
        self.server = server
        self.key = key
        self.expire_time = expire_time
        self.debug = debug
        self.logdupes = True

    @classmethod
    def from_settings(cls, settings):
        """Returns an instance from given settings.

        This uses by default the key ``dupefilter:<timestamp>``. When using the
        ``scrapy_redis.scheduler.Scheduler`` class, this method is not used as
        it needs to pass the spider name in the key.

        Parameters
        ----------
        settings : scrapy.settings.Settings

        Returns
        -------
        RFPDupeFilter
            A RFPDupeFilter instance.


        """
        server = get_redis_from_settings(settings)
        # XXX: This creates one-time key. needed to support to use this
        # class as standalone dupefilter with scrapy's default scheduler
        # if scrapy passes spider on open() method this wouldn't be needed
        # TODO: Use SCRAPY_JOB env as default and fallback to timestamp.
        key = defaults.DUPEFILTER_KEY % {'timestamp': int(time.time())}
        debug = settings.getbool('DUPEFILTER_DEBUG')
        expire_time = settings.get('SCHEDULER_DUPEFILTER_EXPIRE_TIME', defaults.SCHEDULER_DUPEFILTER_EXPIRE_TIME)
        return cls(server, key=key, expire_time=expire_time, debug=debug)

    @classmethod
    def from_crawler(cls, crawler):
        """Returns instance from crawler.

        Parameters
        ----------
        crawler : scrapy.crawler.Crawler

        Returns
        -------
        RFPDupeFilter
            Instance of RFPDupeFilter.

        """
        return cls.from_settings(crawler.settings)

    def request_seen(self, request):
        """Returns True if request was already seen.

        Parameters
        ----------
        request : scrapy.http.Request

        Returns
        -------
        bool

        """
        fp = self.request_fingerprint(request)
        # Use the specified expiration time when the expiration time is specified in the request.meta,
        # otherwise, use the expiration time set in settings.
        expire_time = int(request.meta.get('request_expire_time', self.expire_time))
        # Update the fingerprint information in the zset when the fingerprint does not
        # exist in the zset, or the fingerprint has expired
        not_exsit_in_zset = self.server.zrank(self.key, fp) is None
        fp_expire_time = self.server.zincrby(self.key, 0, fp)
        if not_exsit_in_zset or (int(fp_expire_time) == 0) or (int(fp_expire_time) < int(time.time())):
            self.server.zadd(self.key, {fp: int(time.time()) + expire_time})
            if self.debug:
                if not_exsit_in_zset:
                    self.logger.info("A new fingerprint is added to the collection.")
                else:
                    self.logger.info("The expiration time of a fingerprint has been updated.")
            return False
        else:
            if self.debug:
                self.logger.info("The fingerprint has not expired, skip this request.")
            return True

    def request_fingerprint(self, request):
        """Returns a fingerprint for a given request.

        Parameters
        ----------
        request : scrapy.http.Request

        Returns
        -------
        str

        """
        return request_fingerprint(request)

    @classmethod
    def from_spider(cls, spider):
        settings = spider.settings
        server = get_redis_from_settings(settings)
        dupefilter_key = settings.get("SCHEDULER_DUPEFILTER_KEY", defaults.SCHEDULER_DUPEFILTER_KEY)
        key = dupefilter_key % {'spider': spider.name}
        debug = settings.getbool('DUPEFILTER_DEBUG')
        expire_time = settings.get('SCHEDULER_DUPEFILTER_EXPIRE_TIME', defaults.SCHEDULER_DUPEFILTER_EXPIRE_TIME)
        return cls(server, key=key, expire_time=expire_time, debug=debug)

    def close(self, reason=''):
        """Delete data on close. Called by Scrapy's scheduler.

        Parameters
        ----------
        reason : str, optional

        """
        self.clear()

    def clear(self):
        """Clears fingerprints data."""
        self.server.delete(self.key)

    def log(self, request, spider):
        """Logs given request.

        Parameters
        ----------
        request : scrapy.http.Request
        spider : scrapy.spiders.Spider

        """
        if self.debug:
            msg = "Filtered duplicate request: %(request)s"
            self.logger.debug(msg, {'request': request}, extra={'spider': spider})
        elif self.logdupes:
            msg = ("Filtered duplicate request %(request)s"
                   " - no more duplicates will be shown"
                   " (see DUPEFILTER_DEBUG to show all duplicates)")
            self.logger.debug(msg, {'request': request}, extra={'spider': spider})
            self.logdupes = False

==> scrapy-redis-expiredupefilter-0.2.1.tar.gz: scrapy-redis-expiredupefilter-0.2.1/src/scrapy_redis_expiredupefilter/dupefilter.py <==
"""

"""

import logging
import time
from copy import deepcopy

from scrapy.dupefilters import BaseDupeFilter
from scrapy.utils.request import request_fingerprint
from scrapy.utils.url import canonicalize_url

from . import defaults
from .utils import dict_hash
from .connection import get_redis_from_settings

logger = logging.getLogger(__name__)


def splash_request_fingerprint(request, include_headers=None):
    """ Request fingerprint which takes 'splash' meta key into account """

    fp = request_fingerprint(request, include_headers=include_headers)
    if 'splash' not in request.meta:
        return fp

    splash_options = deepcopy(request.meta['splash'])
    args = splash_options.setdefault('args', {})

    if 'url' in args:
        args['url'] = canonicalize_url(args['url'], keep_fragments=True)

    return dict_hash(splash_options, fp)


# TODO: Rename class to RedisDupeFilter.
class RFPDupeFilter(BaseDupeFilter):
    """Redis-based request duplicates filter.

    This class can also be used with default Scrapy's scheduler.

    """

    logger = logger

    def __init__(self, server, key, expire_time, debug=False):
        """Initialize the duplicates filter.

        Parameters
        ----------
        server : redis.StrictRedis
            The redis server instance.
        key : str
            Redis key Where to store fingerprints.
        debug : bool, optional
            Whether to log filtered requests.

        """
        self.server = server
        self.key = key
        self.expire_time = expire_time
        self.debug = debug
        self.logdupes = True

    @classmethod
    def from_settings(cls, settings):
        """Returns an instance from given settings.

        This uses by default the key ``dupefilter:<timestamp>``. When using the
        ``scrapy_redis.scheduler.Scheduler`` class, this method is not used as
        it needs to pass the spider name in the key.

        Parameters
        ----------
        settings : scrapy.settings.Settings

        Returns
        -------
        RFPDupeFilter
            A RFPDupeFilter instance.


        """
        server = get_redis_from_settings(settings)
        # XXX: This creates one-time key. needed to support to use this
        # class as standalone dupefilter with scrapy's default scheduler
        # if scrapy passes spider on open() method this wouldn't be needed
        # TODO: Use SCRAPY_JOB env as default and fallback to timestamp.
        key = defaults.DUPEFILTER_KEY % {'timestamp': int(time.time())}
        debug = settings.getbool('DUPEFILTER_DEBUG')
        expire_time = settings.get('SCHEDULER_DUPEFILTER_EXPIRE_TIME', defaults.SCHEDULER_DUPEFILTER_EXPIRE_TIME)
        return cls(server, key=key, expire_time=expire_time, debug=debug)

    @classmethod
    def from_crawler(cls, crawler):
        """Returns instance from crawler.

        Parameters
        ----------
        crawler : scrapy.crawler.Crawler

        Returns
        -------
        RFPDupeFilter
            Instance of RFPDupeFilter.

        """
        return cls.from_settings(crawler.settings)

    def request_seen(self, request):
        """Returns True if request was already seen.

        Parameters
        ----------
        request : scrapy.http.Request

        Returns
        -------
        bool

        """
        fp = self.request_fingerprint(request)
        # Use the specified expiration time when the expiration time is specified in the request.meta,
        # otherwise, use the expiration time set in settings.
        expire_time = int(request.meta.get('request_expire_time', self.expire_time))
        # Update the fingerprint information in the zset when the fingerprint does not
        # exist in the zset, or the fingerprint has expired
        not_exsit_in_zset = self.server.zrank(self.key, fp) is None
        fp_expire_time = self.server.zincrby(self.key, 0, fp)
        if not_exsit_in_zset or (int(fp_expire_time) == 0) or (int(fp_expire_time) < int(time.time())):
            self.server.zadd(self.key, {fp: int(time.time()) + expire_time})
            if self.debug:
                if not_exsit_in_zset:
                    self.logger.info("A new fingerprint is added to the collection.")
                else:
                    self.logger.info("The expiration time of a fingerprint has been updated.")
            return False
        else:
            if self.debug:
                self.logger.info("The fingerprint has not expired, skip this request.")
            return True

    def request_fingerprint(self, request):
        """Returns a fingerprint for a given request.
        Compatible with scrapy-splash.

        Parameters
        ----------
        request : scrapy.http.Request

        Returns
        -------
        str

        """
        return splash_request_fingerprint(request)

    @classmethod
    def from_spider(cls, spider):
        settings = spider.settings
        server = get_redis_from_settings(settings)
        dupefilter_key = settings.get("SCHEDULER_DUPEFILTER_KEY", defaults.SCHEDULER_DUPEFILTER_KEY)
        key = dupefilter_key % {'spider': spider.name}
        debug = settings.getbool('DUPEFILTER_DEBUG')
        expire_time = settings.get('SCHEDULER_DUPEFILTER_EXPIRE_TIME', defaults.SCHEDULER_DUPEFILTER_EXPIRE_TIME)
        return cls(server, key=key, expire_time=expire_time, debug=debug)

    def close(self, reason=''):
        """Delete data on close. Called by Scrapy's scheduler.

        Parameters
        ----------
        reason : str, optional

        """
        self.clear()

    def clear(self):
        """Clears fingerprints data."""
        self.server.delete(self.key)

    def log(self, request, spider):
        """Logs given request.

        Parameters
        ----------
        request : scrapy.http.Request
        spider : scrapy.spiders.Spider

        """
        if self.debug:
            msg = "Filtered duplicate request: %(request)s"
            self.logger.debug(msg, {'request': request}, extra={'spider': spider})
        elif self.logdupes:
            msg = ("Filtered duplicate request %(request)s"
                   " - no more duplicates will be shown"
                   " (see DUPEFILTER_DEBUG to show all duplicates)")
            self.logger.debug(msg, {'request': request}, extra={'spider': spider})
            self.logdupes = False
